{
    

       

    . , 1997-2001
  E-mail: roman@sar.nnov.ru
}

unit ExtSysUtils;

interface

{$I EXT.INC}

uses
  SysUtils;

{     .    
   ,      . }

function StrToFloatDef(const S: string; Default: Extended): Extended;

{     .  
  ExceptionErrorMessage  SysUtils.    
   ,    .   
      (..     ). }

function ExceptionErrorMsg(ExceptObject: TObject; ExceptAddr: Pointer): string; 

{     (Exception): 
   ,    . }

function ExceptionMsg(E: Exception): string;

{      , 
     (PartialOK = True)    
   (PartialOK = False). }

function FindParam(const Param: string; PartialOK: Boolean;
  var Value: string): Boolean;

{     . }

function ParamExists(const Param: string; PartialOK: Boolean): Boolean;
function ParamsExists(const Params: array of string;
  PartialOK, CheckAll: Boolean): Boolean;

{   .  ,   
   . ,   -F   
   -FExtUtils.pas   ExtUtils.pas }

function GetParamValue(const Param: string): string;
function GetParamValueDef(const Param: string; const Default: string): string;

{ ,        
      False,   
  .         
   . }

function ArrayAdd(Items: Pointer; Count, ItemSize: Integer;
  Item: Pointer): Boolean;
function ArrayInsert(Items: Pointer; Count, ItemSize: Integer;
  Index: Integer; Item: Pointer): Boolean;
function ArrayDelete(Items: Pointer; Count, ItemSize: Integer;
  Index: Integer; Item: Pointer): Boolean;

function ArrayExchange(Items: Pointer; Count, ItemSize: Integer;
  Index1, Index2: Integer): Boolean;
function ArrayMove(Items: Pointer; Count, ItemSize: Integer;
  CurIndex, NewIndex: Integer): Boolean;

type
  TArrayCompare = function(Item1, Item2: Pointer): Integer;

procedure ArraySort(Items: Pointer; Count, ItemSize: Integer;
  Compare: TArrayCompare);

function ArrayFind(Items: Pointer; Count, ItemSize: Integer;
  Item: Pointer; Compare: TArrayCompare; var Index: Integer): Boolean;
function ArrayIndexOf(Items: Pointer; Count, ItemSize: Integer;
  Item: Pointer; Compare: TArrayCompare): Integer;

{  .      nil.  }

procedure FreeMemAndNil(var P);

{$IFNDEF EXT_D5_UP}

procedure FreeAndNil(var Obj);

{$ENDIF}

implementation

uses
  ExtStrUtils;

{      }

function StrToFloatDef(const S: string; Default: Extended): Extended;
begin
  if not TextToFloat(PChar(S), Result, fvExtended) then Result := Default;
end;

{     }

function ExceptionMsg(E: Exception): string;
begin
  Result := RemoveLineBreaks(TrimCharRight(E.Message, '.'));
end;

function ExceptionErrorMsg(ExceptObject: TObject; ExceptAddr: Pointer): string;
var
  Buffer: array[0..1023] of Char;
begin
  FillChar(Buffer, SizeOf(Buffer), 0);
  ExceptionErrorMessage(ExceptObject, ExceptAddr, Buffer, SizeOf(Buffer));
  Result := Buffer;
end;

{   }

function FindParam(const Param: string; PartialOK: Boolean;
  var Value: string): Boolean;
var
  I, P: Integer;
begin
  {     }
  for I := 1 to ParamCount do
  begin
    {   }
    Value := ParamStr(I);
    {    }
    if PartialOK then
    begin
      {     }
      P := CasePos(Param, Value, False);
      if P = 1 then
      begin
        {   -  }
        Result := True;
        Exit;
      end;
    end
    else
      {  }
      if AnsiCompareText(Param, Value) = 0 then
      begin
        {   -  }
        Result := True;
        Exit;
      end;
  end;
  {   }
  Value := '';
  Result := False;
end;                                            

{      }

function ParamExists(const Param: string; PartialOK: Boolean): Boolean;
var
  Value: string;
begin
  Result := FindParam(Param, PartialOK, Value);
end;

function ParamsExists(const Params: array of string;
  PartialOK, CheckAll: Boolean): Boolean;
var
  I, Count: Integer;
begin
  Count := 0;
  {   }
  for I := Low(Params) to High(Params) do
    if ParamExists(Params[I], PartialOK) then Inc(Count);
  {  }
  if CheckAll then
    Result := Count = High(Params) - Low(Params) + 1
  else
   Result := Count > 0;
end;

{    }

function GetParamValue(const Param: string): string;
begin
  Result := GetParamValueDef(Param, '');
end;

function GetParamValueDef(const Param: string; const Default: string): string;
var
  Value: string;
begin
  if FindParam(Param, True, Value) then
    Result := Copy(Value, Length(Param) + 1, MaxInt)
  else
    Result := Default;
end;

{ ,        
     }

function ArrayAdd(Items: Pointer; Count, ItemSize: Integer;
  Item: Pointer): Boolean;
begin
  Result := ArrayInsert(Items, Count, ItemSize, Count, Item);
end;

function ArrayInsert(Items: Pointer; Count, ItemSize: Integer;
  Index: Integer; Item: Pointer): Boolean;
var
  PI, PI1: PChar;
begin
  {   }
  if (Index < 0) or (Index > Count) then
  begin
    Result := False;
    Exit;
  end;
  {    Index   }
  PI := PChar(Items) + Index * ItemSize;
  PI1 := PChar(Items) + (Index + 1) * ItemSize;
  {  }
  if Index < Count then Move(PI^, PI1^, (Count - Index) * ItemSize);
  if Item <> nil then Move(Item^, PI^, ItemSize);
  {   }
  Result := True;
end;

function ArrayDelete(Items: Pointer; Count, ItemSize: Integer;
  Index: Integer; Item: Pointer): Boolean;
var
  PI, PI1: PChar;
begin
  {   }
  if (Index < 0) or (Index >= Count) then
  begin
    Result := False;
    Exit;
  end;
  {    Index   }
  PI := PChar(Items) + Index * ItemSize;
  PI1 := PChar(Items) + (Index + 1) * ItemSize;
  {  ,  }
  if Item <> nil then Move(PI^, Item^, ItemSize);
  if Index < Count then System.Move(PI1^, PI^, (Count - Index) * ItemSize);
  {   }
  Result := True;
end;

function ArrayExchange(Items: Pointer; Count, ItemSize: Integer;
  Index1, Index2: Integer): Boolean;
var
  PI1, PI2: Pointer;
  Item: Pointer;
begin
  Result := True;
  {       }
  if Index1 <> Index2 then
  begin
    {   }
    if (Index1 < 0) or (Index1 >= Count) or (Index2 < 0) or (Index2 >= Count) then
    begin
      Result := False;
      Exit;
    end;
    {    }
    GetMem(Item, ItemSize);
    try
      {    Index1 Index2 }
      PI1 := PChar(Items) + Index1 * ItemSize;
      PI2 := PChar(Items) + Index2 * ItemSize;
      {   }
      Move(PI1^, Item^, ItemSize);
      Move(PI2^, PI1^, ItemSize);
      Move(Item^, PI2^, ItemSize);
    finally
      FreeMem(Item, ItemSize);
    end;
  end;
end;

function ArrayMove(Items: Pointer; Count, ItemSize: Integer;
  CurIndex, NewIndex: Integer): Boolean;
var
  Item: Pointer;
begin
  Result := True;
  {     }
  if CurIndex <> NewIndex then
  begin
    {   }
    if (NewIndex < 0) or (NewIndex >= Count) then
    begin
      Result := False;
      Exit;
    end;
    {    }
    GetMem(Item, ItemSize);
    try
      {  }
      ArrayDelete(Items, Count, ItemSize, CurIndex, Item);
      ArrayInsert(Items, Count, ItemSize, NewIndex, Item);
    finally
      FreeMem(Item, ItemSize);
    end;
  end;
end;

procedure ArrayQuickSort(Items, Temp: Pointer; Count, ItemSize, L, R: Integer;
  SCompare: TArrayCompare);
var
  I, J: Integer;
  P, ItemI, ItemJ: Pointer;
begin
  repeat
    I := L;
    J := R;
    P := PChar(Items) + ((L + R) shr 1) * ItemSize;
    repeat
      while SCompare(PChar(Items) + I * ItemSize, P) < 0 do Inc(I);
      while SCompare(PChar(Items) + J * ItemSize, P) > 0 do Dec(J);
      if I <= J then
      begin
        ItemI := PChar(Items) + I * ItemSize;
        ItemJ := PChar(Items) + J * ItemSize;
        Move(ItemI^, Temp^, ItemSize);
        Move(ItemJ^, ItemI^, ItemSize);
        Move(Temp^, ItemJ^, ItemSize);
        if P = ItemI then P := ItemJ
        else if P = ItemJ then P := ItemI;
        Inc(I);
        Dec(J);
      end;
    until I > J;
    if L < J then
      ArrayQuickSort(Items, Temp, Count, ItemSize, L, J, SCompare);
    L := I;
  until I >= R;
end;

procedure ArraySort(Items: Pointer; Count, ItemSize: Integer;
  Compare: TArrayCompare);
var
  Temp: Pointer;
begin
  if Count > 1 then
  begin
    GetMem(Temp, ItemSize);
    try
      ArrayQuickSort(Items, Temp, Count, ItemSize, 0, Count - 1, Compare);
    finally
      FreeMem(Temp);
    end;
  end;
end;

function ArrayFind(Items: Pointer; Count, ItemSize: Integer;
  Item: Pointer; Compare: TArrayCompare; var Index: Integer): Boolean;
var
  L, H, I, C: Integer;
begin
  Result := False;
  L := 0;
  H := Count - 1;
  while L <= H do
  begin
    I := (L + H) shr 1;
    C := Compare(PChar(Items) + I * ItemSize, Item);
    if C < 0 then L := I + 1 else
    begin
      H := I - 1;
      if C = 0 then
      begin
        Result := True;
        // if Duplicates <> dupAccept then L := I;
      end;
    end;
  end;
  Index := L;
end;

function ArrayIndexOf(Items: Pointer; Count, ItemSize: Integer;
  Item: Pointer; Compare: TArrayCompare): Integer;
begin
  for Result := 0 to Count - 1 do
    if Compare(PChar(Items) + Result * ItemSize, Item) = 0 then Exit;
  Result := -1;
end;

{  .      nil.  }

procedure FreeMemAndNil(var P);
var
  Mem: Pointer;
begin
  Mem := Pointer(P);
  Pointer(P) := nil;
  if Mem <> nil then FreeMem(Mem);
end;

{$IFNDEF EXT_D5_UP}

procedure FreeAndNil(var Obj);
var
  P: TObject;
begin
  P := TObject(Obj);
  TObject(Obj) := nil;
  P.Free;
end;

{$ENDIF}

end.
